/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.classfile;

import org.apache.bcel.support.PushedDown;
import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.util.CheckClassAdapter;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class JavaClass extends AccessFlags {

	/**
	 * Wrappee field.
	 */
	@WrapperMember
	private ClassNode classNode = null;
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	public JavaClass(ClassNode node) {
		setClassNode(node);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param node
	 */
	@WrapperMember
	protected ClassNode setClassNode(ClassNode node) {
		this.classNode = node;
		WrapperRecycler.add(this, node);
		return node;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	public ClassNode getClassNode() {
		return classNode;
	}

	
	public JavaClass() {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * Checks that this java class represents a class (as opposed to an interface).
	 *
	 * @return
	 */
	public final boolean isClass() {
		return (classNode.access & Opcodes.ACC_INTERFACE) == 0;
	}

	/**
	 * Returns the byte representation of this class.
	 *
	 * @return
	 */
	public byte[] getBytes() {
		ClassWriter writer = new ClassWriter(ClassWriter.COMPUTE_MAXS);
		this.classNode.accept(new CheckClassAdapter(writer, false));
		//this.classNode.accept(writer);
		return writer.toByteArray();
	}

	/**
	 * Originally on super type.
	 *
	 * @return
	 */
	@Override
	@PushedDown
	public boolean isPublic() {
		throw new UnsupportedOperationException();
	}

	/**
	 * Originally on super type.
	 *
	 * @param arg0
	 */
	@Override
	@PushedDown
	public void setAccessFlags(int arg0) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Originally on super type.
	 *
	 * @return
	 */
	@Override
	@PushedDown
	public int getAccessFlags() {
		throw new UnsupportedOperationException();
	}
	

	/**
	 * Computes the major portion of the class version.
	 *
	 * @return
	 */
	public int getMajor() {
		throw new UnsupportedOperationException();
    }
    
	/**
	 * Computes the minor portion of the class version.
	 *
	 * @return
	 */
	public int getMinor() {
		throw new UnsupportedOperationException();
    }
}
