/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.classfile;

import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.support.WrapperMember;
import org.objectweb.asm.tree.LineNumberNode;


/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public final class LineNumberTable extends Attribute {

	private final InstructionList list;
	
	/**
	 * TODO Comment!!
	 *
	 */
	@WrapperMember
	public LineNumberTable(InstructionList list) {
		this.list = list;
	}
	
	/**
	 * It seems impossible to easily get this information using ASM when the
	 * application is constructing a method (it's fine when parsing). We need
	 * to write a visitor between writing or reading to get the source line
	 * information, as well as any other attribute information.
	 *
	 * @param pos
	 * @return
	 */
	public int getSourceLine(int pos) {
		
		// linear search is good enough
		InstructionHandle h = list.getStart();
		InstructionHandle last = h;
		while(null != h && h.getPosition() <= pos) {
			last = h;
			h = h.getNext();
		}
		
		LineNumberNode ln = last.getLineNumber();
		if (null == ln)
			return -1;
		return ln.line;
	}
}
