/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import java.util.Map;

import org.apache.bcel.support.WrapperMember;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public abstract class BranchInstruction extends Instruction implements InstructionTargeter {
	
	BranchInstruction() {
	}

	/**
	 * Sets the target of this branch.
	 *
	 * @param target
	 */
	public void setTarget(InstructionHandle target) {
		this.setTargetLabel(this.getNode(), target.ensureLabel());
	}
	
	@WrapperMember
	void setTarget(InstructionHandle source, InstructionHandle target) {
		this.setTargetLabel(source.getNode(), target.ensureLabel());
	}
	
	/**
	 * Gets the target of this branch.
	 *
	 * @return
	 */
	public InstructionHandle getTarget() {
		// TODO - make sure we get the right handle for the label
		LabelNode label = getTargetLabel();
		return this.handles.get(0).list.getHandle(label);
	}
	
	@Override
	@WrapperMember
	InstructionHandle createHandle(AbstractInsnNode node) {
		return BranchHandle.createHandle(node);
	}
	
	/**
	 * Gets the label that is the target of this branch.
	 *
	 * @return
	 */
	@WrapperMember
	abstract LabelNode getTargetLabel();
	
	/**
	 * Sets the label that is the target of this branch into this node.
	 *
	 * @param label
	 */
	@WrapperMember
	abstract void setTargetLabel(AbstractInsnNode node, LabelNode label);
	
	/**
	 * Helper to register the targeter relationship between this branch instruction
	 * and target instructions.
	 *
	 * @param target
	 */
	@WrapperMember
	protected void registerTargeter(InstructionHandle target) {
		if (null != target)
			target.addTargeter(this);
		setTarget(target);
	}
	

	/**
	 * Most BranchInstructions target a JumpInsnNode. If not, they must override this method.
	 *
	 * @param node
	 * @param map
	 * @return
	 */
	@Override
	@WrapperMember
	Instruction rewrite(AbstractInsnNode node, Map<LabelNode, InstructionHandle> map) {
		JumpInsnNode i = (JumpInsnNode) node;
		InstructionHandle handle = map.get(i.label);
		
		i.label = map.get(i.label).ensureLabel();
		registerTargeter(handle);
		return this;
	}
}
