/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.support.WrapperMember;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.InsnNode;


/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class ICONST extends Instruction {

	/**
	 * Creates a constant. Accepts only values from -1 to 5.
	 * 
	 * @param value
	 */
	public ICONST(int value) {
		switch (value) {
		case -1:
			setNode(new InsnNode(Opcodes.ICONST_M1));
			break;
		case 0:
			setNode(new InsnNode(Opcodes.ICONST_0));
			break;
		case 1:
			setNode(new InsnNode(Opcodes.ICONST_1));
			break;
		case 2:
			setNode(new InsnNode(Opcodes.ICONST_2));
			break;
		case 3:
			setNode(new InsnNode(Opcodes.ICONST_3));
			break;
		case 4:
			setNode(new InsnNode(Opcodes.ICONST_4));
			break;
		case 5:
			setNode(new InsnNode(Opcodes.ICONST_5));
			break;
		default:
			throw new ClassGenException(
					"ICONST can be used only for value between -1 and 5: "
							+ value);
		}
	}
	
	@WrapperMember
	ICONST() {	
	}

	/**
	 * Shallow clone of this instruction.
	 *
	 * @return
	 */
	@WrapperMember
	public Instruction cloneInstruction() {
		ICONST i = new ICONST();
		i.setNode(new InsnNode(((InsnNode) getNode()).getOpcode()));
		return i;
	}
	
	@Override
	@WrapperMember
	public AbstractInsnNode cloneNode() {
		return new InsnNode(((InsnNode) getNode()).getOpcode());
	}
}
