/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.Constants;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public interface InstructionConstants {
	
	// Predefined instructions (only for the instructions used so far)
	public static final Instruction NOP = new NOP();
	public static final Instruction ACONST_NULL = new ACONST_NULL();
	public static final Instruction ICONST_M1 = new ICONST(-1);
	public static final Instruction ICONST_0 = new ICONST(0);
	public static final Instruction ICONST_1 = new ICONST(1);
	public static final Instruction ICONST_2 = new ICONST(2);
	public static final Instruction ICONST_3 = new ICONST(3);
	public static final Instruction ICONST_4 = new ICONST(4);
	public static final Instruction ICONST_5 = new ICONST(5);
	public static final StackInstruction DUP = new DUP();
	public static final Instruction DUP2 = new DUP2();
	
	public static final Instruction IADD = new IADD();
	public static final ArithmeticInstruction ISUB = new ISUB();
	public static final ArithmeticInstruction IMUL = new IMUL();
	public static final ReturnInstruction IRETURN = new IRETURN();
	public static final ReturnInstruction ARETURN = new ARETURN();
	public static final ReturnInstruction RETURN = new RETURN();
	public static final Instruction ATHROW = new ATHROW();

	public static final LocalVariableInstruction THIS = new ALOAD(0);
	public static final LocalVariableInstruction ALOAD_0 = THIS;
	public static final LocalVariableInstruction ALOAD_1 = new ALOAD(1);
	public static final LocalVariableInstruction ALOAD_2 = new ALOAD(2);
	public static final LocalVariableInstruction ILOAD_0 = new ILOAD(0);
	public static final LocalVariableInstruction ILOAD_1 = new ILOAD(1);
	public static final LocalVariableInstruction ILOAD_2 = new ILOAD(2);
	public static final Instruction IALOAD = new IALOAD();
	
	public static final LocalVariableInstruction ASTORE_0 = new ASTORE(0);
	public static final LocalVariableInstruction ASTORE_1 = new ASTORE(1);
	public static final LocalVariableInstruction ASTORE_2 = new ASTORE(2);
	public static final LocalVariableInstruction ISTORE_0 = new ISTORE(0);
	public static final LocalVariableInstruction ISTORE_1 = new ISTORE(1);
	public static final LocalVariableInstruction ISTORE_2 = new ISTORE(2);
	public static final Instruction IASTORE = new IASTORE();
    
    /**
     * Singletons for instructions that can be completely determined statically.
     */
    public static final Instruction[] INSTRUCTIONS = new Instruction[256];
    
    public static final Class<?>[] INSTRUCTION_WRAPPERS = new Class<?>[256];
    
    /**
     * An initializer for the static arrays.
     */
    static final ArrayInitializer initializer = new ArrayInitializer();
    static class ArrayInitializer {

    	ArrayInitializer() {
    		initInstructions();
    		initWrappers();
    	}
    	
    	void initInstructions() {
         
            INSTRUCTIONS[Constants.NOP] = NOP;
            INSTRUCTIONS[Constants.ACONST_NULL] = ACONST_NULL;
            INSTRUCTIONS[Constants.ICONST_M1] = ICONST_M1;
            INSTRUCTIONS[Constants.ICONST_0] = ICONST_0;
            INSTRUCTIONS[Constants.ICONST_1] = ICONST_1;
            INSTRUCTIONS[Constants.ICONST_2] = ICONST_2;
            INSTRUCTIONS[Constants.ICONST_3] = ICONST_3;
            INSTRUCTIONS[Constants.ICONST_4] = ICONST_4;
            INSTRUCTIONS[Constants.ICONST_5] = ICONST_5;
            INSTRUCTIONS[Constants.IALOAD] = IALOAD;
            INSTRUCTIONS[Constants.IASTORE] = IASTORE;
            INSTRUCTIONS[Constants.DUP] = DUP;
            INSTRUCTIONS[Constants.DUP2] = DUP2;
            INSTRUCTIONS[Constants.IADD] = IADD;
            INSTRUCTIONS[Constants.ISUB] = ISUB;
            INSTRUCTIONS[Constants.IMUL] = IMUL;
            INSTRUCTIONS[Constants.IRETURN] = IRETURN;
            INSTRUCTIONS[Constants.ARETURN] = ARETURN;
            INSTRUCTIONS[Constants.RETURN] = RETURN;
            INSTRUCTIONS[Constants.ATHROW] = ATHROW;
        }
    	
    	void initWrappers() {

    		INSTRUCTION_WRAPPERS[Constants.ACONST_NULL] = org.apache.bcel.generic.ACONST_NULL.class;
    		INSTRUCTION_WRAPPERS[Constants.ALOAD] = org.apache.bcel.generic.ALOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ALOAD_0] = org.apache.bcel.generic.ALOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ALOAD_1] = org.apache.bcel.generic.ALOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ALOAD_2] = org.apache.bcel.generic.ALOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ALOAD_3] = org.apache.bcel.generic.ALOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.IALOAD] = org.apache.bcel.generic.IALOAD.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.ASTORE] = org.apache.bcel.generic.ASTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ASTORE_0] = org.apache.bcel.generic.ASTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ASTORE_1] = org.apache.bcel.generic.ASTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ASTORE_2] = org.apache.bcel.generic.ASTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ASTORE_3] = org.apache.bcel.generic.ASTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.IASTORE] = org.apache.bcel.generic.IASTORE.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.ATHROW] = org.apache.bcel.generic.ATHROW.class;
    		INSTRUCTION_WRAPPERS[Constants.BIPUSH] = org.apache.bcel.generic.BIPUSH.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.CHECKCAST] = org.apache.bcel.generic.CHECKCAST.class;
    		INSTRUCTION_WRAPPERS[Constants.DUP] = org.apache.bcel.generic.DUP.class;
    		INSTRUCTION_WRAPPERS[Constants.DUP2] = org.apache.bcel.generic.DUP2.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.GETFIELD] = org.apache.bcel.generic.GETFIELD.class;
    		INSTRUCTION_WRAPPERS[Constants.PUTFIELD] = org.apache.bcel.generic.PUTFIELD.class;
    		INSTRUCTION_WRAPPERS[Constants.GETSTATIC] = org.apache.bcel.generic.GETSTATIC.class;
    		INSTRUCTION_WRAPPERS[Constants.PUTSTATIC] = org.apache.bcel.generic.PUTSTATIC.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.GOTO] = org.apache.bcel.generic.GOTO.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.ICONST_0] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_1] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_2] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_3] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_4] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_5] = org.apache.bcel.generic.ICONST.class;
    		INSTRUCTION_WRAPPERS[Constants.ICONST_M1] = org.apache.bcel.generic.ICONST.class;
    		
    		
    		INSTRUCTION_WRAPPERS[Constants.IFEQ] = org.apache.bcel.generic.IFEQ.class;
    		INSTRUCTION_WRAPPERS[Constants.IFGT] = org.apache.bcel.generic.IFGT.class;
    		INSTRUCTION_WRAPPERS[Constants.IFLE] = org.apache.bcel.generic.IFLE.class;
    		INSTRUCTION_WRAPPERS[Constants.IFNE] = org.apache.bcel.generic.IFNE.class;
    		INSTRUCTION_WRAPPERS[Constants.IF_ICMPLT] = org.apache.bcel.generic.IF_ICMPLT.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.ILOAD] = org.apache.bcel.generic.ILOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ILOAD_0] = org.apache.bcel.generic.ILOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ILOAD_1] = org.apache.bcel.generic.ILOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ILOAD_2] = org.apache.bcel.generic.ILOAD.class;
    		INSTRUCTION_WRAPPERS[Constants.ILOAD_3] = org.apache.bcel.generic.ILOAD.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.IINC] = org.apache.bcel.generic.IINC.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.IMUL] = org.apache.bcel.generic.IMUL.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.INVOKEINTERFACE] = org.apache.bcel.generic.INVOKEINTERFACE.class;
    		INSTRUCTION_WRAPPERS[Constants.INVOKESPECIAL] = org.apache.bcel.generic.INVOKESPECIAL.class;
    		INSTRUCTION_WRAPPERS[Constants.INVOKESTATIC] = org.apache.bcel.generic.INVOKESTATIC.class;
    		INSTRUCTION_WRAPPERS[Constants.INVOKEVIRTUAL] = org.apache.bcel.generic.INVOKEVIRTUAL.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.IRETURN] = org.apache.bcel.generic.IRETURN.class;
    		INSTRUCTION_WRAPPERS[Constants.ARETURN] = org.apache.bcel.generic.ARETURN.class;
    		INSTRUCTION_WRAPPERS[Constants.ISTORE] = org.apache.bcel.generic.ISTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ISTORE_0] = org.apache.bcel.generic.ISTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ISTORE_1] = org.apache.bcel.generic.ISTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ISTORE_2] = org.apache.bcel.generic.ISTORE.class;
    		INSTRUCTION_WRAPPERS[Constants.ISTORE_3] = org.apache.bcel.generic.ISTORE.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.IADD] = org.apache.bcel.generic.IADD.class;
    		INSTRUCTION_WRAPPERS[Constants.ISUB] = org.apache.bcel.generic.ISUB.class;
    		INSTRUCTION_WRAPPERS[Constants.JSR] = org.apache.bcel.generic.JSR.class;
    		INSTRUCTION_WRAPPERS[Constants.LOOKUPSWITCH] = org.apache.bcel.generic.LOOKUPSWITCH.class;
    		INSTRUCTION_WRAPPERS[Constants.NEW] = org.apache.bcel.generic.NEW.class;
    		INSTRUCTION_WRAPPERS[Constants.NEWARRAY] = org.apache.bcel.generic.NEWARRAY.class;
    		INSTRUCTION_WRAPPERS[Constants.NOP] = org.apache.bcel.generic.NOP.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.RET] = org.apache.bcel.generic.RET.class;
    		INSTRUCTION_WRAPPERS[Constants.RETURN] = org.apache.bcel.generic.RETURN.class;
    		INSTRUCTION_WRAPPERS[Constants.SIPUSH] = org.apache.bcel.generic.SIPUSH.class;
    		
    		INSTRUCTION_WRAPPERS[Constants.LDC] = org.apache.bcel.generic.LDC.class;
    		
        }
    }

}
