/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.Constants;


/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class InstructionFactory implements InstructionConstants {
	
	public InstructionFactory(ClassGen cg, ConstantPoolGen cp) {
	}

	/**
	 * Create an instruction factory initialized with a ClassGen.
	 *
	 * @param classGen
	 */
	public InstructionFactory(ClassGen classGen) {
	}

	/**
	 * 
	 * TODO Comment!!
	 *
	 * @param type
	 * @return
	 */
	public static ReturnInstruction createReturn(Type type) {
		switch(type.getType().getSort()) {
		case org.objectweb.asm.Type.BOOLEAN:
		case org.objectweb.asm.Type.CHAR:
		case org.objectweb.asm.Type.BYTE:
		case org.objectweb.asm.Type.SHORT:
		case org.objectweb.asm.Type.INT:
            return IRETURN;
			
		case org.objectweb.asm.Type.FLOAT:
            //return new FRETURN;
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.DOUBLE:
            //return new DRETURN;
    		throw new UnsupportedOperationException();
            
		case org.objectweb.asm.Type.LONG:
            //return new LRETURN;
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.ARRAY:
		case org.objectweb.asm.Type.OBJECT:
            return ARETURN;

		case org.objectweb.asm.Type.VOID:
			return RETURN;
			
        default:
            throw new RuntimeException("Invalid type " + type);
		}
	}

	public static LocalVariableInstruction createStore(Type type, int index) {
		switch(type.getType().getSort()) {
		case org.objectweb.asm.Type.BOOLEAN:
		case org.objectweb.asm.Type.CHAR:
		case org.objectweb.asm.Type.BYTE:
		case org.objectweb.asm.Type.SHORT:
		case org.objectweb.asm.Type.INT:
            return new ISTORE(index);
			
		case org.objectweb.asm.Type.FLOAT:
            //return new FSTORE(index);
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.DOUBLE:
            //return new DSTORE(index);
    		throw new UnsupportedOperationException();
            
		case org.objectweb.asm.Type.LONG:
            //return new LSTORE(index);
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.ARRAY:
		case org.objectweb.asm.Type.OBJECT:
            return new ASTORE(index);
			
        default:
            throw new RuntimeException("Invalid type " + type);
		}
	}

	/**
	 * 
	 * TODO Comment!!
	 *
	 * @param type
	 * @param index
	 * @return
	 */
	public static LocalVariableInstruction createLoad(Type type, int index) {
		switch(type.getType().getSort()) {
		case org.objectweb.asm.Type.BOOLEAN:
		case org.objectweb.asm.Type.CHAR:
		case org.objectweb.asm.Type.BYTE:
		case org.objectweb.asm.Type.SHORT:
		case org.objectweb.asm.Type.INT:
            return new ILOAD(index);
			
		case org.objectweb.asm.Type.FLOAT:
            //return new FLOAD(index);
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.DOUBLE:
            //return new DLOAD(index);
    		throw new UnsupportedOperationException();
            
		case org.objectweb.asm.Type.LONG:
            //return new LLOAD(index);
			throw new UnsupportedOperationException();
			
		case org.objectweb.asm.Type.ARRAY:
		case org.objectweb.asm.Type.OBJECT:
            return new ALOAD(index);

        default:
            throw new RuntimeException("Invalid type " + type);
		}
	}

	/**
	 * 
	 * TODO Comment!!
	 *
	 * @param arg0
	 * @return
	 */
	public NEW createNew(String s) {
        //return createNew(new ObjectType(s));
        return new NEW(new ObjectType(s).getType().getInternalName());
	}

	/**
	 * Creates the appropriate invoke instruction based on the kind.
	 *
	 * @param class_name
	 * @param name
	 * @param ret_type
	 * @param arg_types
	 * @param kind
	 * @return
	 */
	public InvokeInstruction createInvoke(String class_name, String name, Type ret_type, Type[] arg_types, short kind) {
		
		String internalName = Type.getInternalName(class_name);
		switch (kind) {
		case Constants.INVOKESPECIAL:
			return new INVOKESPECIAL(internalName, name, Type.createMethodDescriptor(ret_type, arg_types));
		case Constants.INVOKEVIRTUAL:
			return new INVOKEVIRTUAL(internalName, name, Type.createMethodDescriptor(ret_type, arg_types));
		case Constants.INVOKESTATIC:
			return new INVOKESTATIC(internalName, name, Type.createMethodDescriptor(ret_type, arg_types));
		case Constants.INVOKEINTERFACE:
			return new INVOKEINTERFACE(internalName, name, Type.createMethodDescriptor(ret_type, arg_types));
		default:
			throw new RuntimeException("Oops: Unknown invoke kind:" + kind);
		}
	}

	/**
	 * Creates an instruction to check a cast for this type.
	 *
	 * @param type
	 * @return
	 */
	public CHECKCAST createCheckCast(ReferenceType type) {
		return new CHECKCAST(type.getType().getInternalName());
		/*
		if (type instanceof ArrayType) {
            return new CHECKCAST(cp.addArrayClass((ArrayType) type));
        } else {
            return new CHECKCAST(cp.addClass((ObjectType) type));
        }*/
	}

	/**
	 * Creates the appropriate field access instruction based on the kind.
	 *
	 * @param class_name
	 * @param name
	 * @param type
	 * @param kind
	 * @return
	 */
	public FieldInstruction createFieldAccess(String class_name, String name, Type type, short kind) {
		
		switch (kind) {
        case Constants.GETFIELD:
            return new GETFIELD(class_name, name, type.getType().getDescriptor());
        case Constants.PUTFIELD:
            return new PUTFIELD(class_name, name, type.getType().getDescriptor());
        case Constants.GETSTATIC:
            return new GETSTATIC(class_name, name, type.getType().getDescriptor());
        case Constants.PUTSTATIC:
            return new PUTSTATIC(class_name, name, type.getType().getDescriptor());
        default:
            throw new RuntimeException("Oops: Unknown getfield kind:" + kind);
    }
	}
}
