/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LineNumberNode;

/**
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class LineNumberGen implements InstructionTargeter {
	
	/**
	 * Wrappee field.
	 */
	@WrapperMember
	private LineNumberNode node = null; 
	
	/**
	 * Stateful adapter.
	 */
	@WrapperMember
	InstructionHandle handle = null;
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	public LineNumberGen(LineNumberNode node) {
		setNode(node);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param node
	 */
	@WrapperMember
	protected LineNumberNode setNode(LineNumberNode node) {
		this.node = node;
		WrapperRecycler.add(this, node);
		return node;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected LineNumberNode getNode() {
		return node;
	}
	
	
	public LineNumberGen(InstructionHandle ih, int src_line) {
		this.handle = ih;
		LineNumberNode node = new LineNumberNode(src_line, null);
		setNode(node);
	}

	/**
	 * Gets the source code line number indicated by this object.
	 *
	 * @return
	 */
	public int getSourceLine() {
		return getNode().line;
	}
	   
	@WrapperMember
	public void setInstructionHandle(InstructionHandle ih) {
		this.handle = ih;
		ih.addTargeter(this);
	}
	
	
	/**
	 * Returns all labels that are targets of this instruction.
	 *
	 * @return
	 */
	@WrapperMember
	public LabelNode[] getTargetLabels() {
		LineNumberNode n = (LineNumberNode) getNode();
		return new LabelNode[] {n.start};
	}
}
