/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import java.util.Arrays;

import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;



/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public abstract class Type {
	// primitives and void
	public final static org.apache.bcel.generic.BasicType VOID = new BasicType(org.objectweb.asm.Type.VOID_TYPE);
	public final static org.apache.bcel.generic.BasicType INT = new BasicType(org.objectweb.asm.Type.INT_TYPE);
	public final static org.apache.bcel.generic.BasicType BOOLEAN = new BasicType(org.objectweb.asm.Type.BOOLEAN_TYPE);
	public final static org.apache.bcel.generic.BasicType CHAR = new BasicType(org.objectweb.asm.Type.CHAR_TYPE);
	public final static org.apache.bcel.generic.BasicType BYTE = new BasicType(org.objectweb.asm.Type.BYTE_TYPE);
	public final static org.apache.bcel.generic.BasicType SHORT = new BasicType(org.objectweb.asm.Type.SHORT_TYPE);
	public final static org.apache.bcel.generic.BasicType FLOAT = new BasicType(org.objectweb.asm.Type.FLOAT_TYPE);
	public final static org.apache.bcel.generic.BasicType LONG = new BasicType(org.objectweb.asm.Type.LONG_TYPE);
	public final static org.apache.bcel.generic.BasicType DOUBLE = new BasicType(org.objectweb.asm.Type.DOUBLE_TYPE);
	
    	 
	// pre-defined types
	public final static org.apache.bcel.generic.ObjectType OBJECT = new ObjectType("java.lang.Object");
	public final static org.apache.bcel.generic.ObjectType STRING = new ObjectType("java.lang.String");

	// for methods with no arguments
	public final static org.apache.bcel.generic.Type[] NO_ARGS = new Type[0];
		
	/**
	 * Wrappee field.
	 */
	@WrapperMember
	private org.objectweb.asm.Type type = null;
	
	/**
	 * Wrapper constructor.
	 *
	 * @param type
	 */
	@WrapperMember
	Type(org.objectweb.asm.Type type) {
		setType(type);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param type
	 */
	@WrapperMember
	protected org.objectweb.asm.Type setType(org.objectweb.asm.Type type) {
		this.type = type;
		WrapperRecycler.add(this, type);
		return type;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected org.objectweb.asm.Type getType() {
		return type;
	}
	
	/**
	 * Wrapper helper method, nor part of the API.
	 * 
	 * @param types
	 * @return
	 */
	@WrapperMember
	public static org.objectweb.asm.Type[] convertTypes(Type[] types) {
		org.objectweb.asm.Type[] asmTypes = new org.objectweb.asm.Type[types.length];
		for (int i = 0; i < types.length; i++)
			asmTypes[i] = types[i].getType();
		return asmTypes;
	}
	
	@WrapperMember
	public static Type[] convertTypes(org.objectweb.asm.Type[] asmTypes) {
		Type[] types = new Type[asmTypes.length];
		for (int i = 0; i < types.length; i++)
			types[i] = convertType(asmTypes[i]);
		return types;
	}
	
	@WrapperMember
	public static Type convertType(org.objectweb.asm.Type asmType) {
		switch(asmType.getSort()) {
		case org.objectweb.asm.Type.VOID:
			return Type.VOID;
		case org.objectweb.asm.Type.BOOLEAN:
			return Type.BOOLEAN;
		case org.objectweb.asm.Type.INT:
			return Type.INT;
         case org.objectweb.asm.Type.CHAR:
        	 return Type.CHAR;
         case org.objectweb.asm.Type.BYTE:
        	 return Type.BYTE;
         case org.objectweb.asm.Type.SHORT:
        	 return Type.SHORT;
         case org.objectweb.asm.Type.FLOAT:
        	 return Type.FLOAT;
         case org.objectweb.asm.Type.LONG:
        	 return Type.LONG;
         case org.objectweb.asm.Type.DOUBLE:
        	 return Type.DOUBLE;
         case org.objectweb.asm.Type.ARRAY:
		case org.objectweb.asm.Type.OBJECT:
			return new ObjectType(asmType.getClassName());
		}
		return null;
	}
	
	/**
	 * Creates an ASM method descriptor for a method with this return type and
	 * parameter types.
	 * 
	 * @param ret_type
	 * @param arg_types
	 * @return
	 */
	@WrapperMember
	public static String createMethodDescriptor(Type ret_type, Type[] arg_types) {
		StringBuilder b = new StringBuilder("(");
		for (int i = 0; i < arg_types.length; i++) {
			b.append(arg_types[i].getType().getDescriptor());
		}
		b.append(")").append(ret_type.getType().getDescriptor());
		return b.toString();
	}
	
	/**
	 * Creates a type that represents an array with this component type and this number
	 * of dimensions.
	 *
	 * @param componentType
	 * @param dimensions
	 * @return
	 */
	public static org.objectweb.asm.Type getArrayType(Type componentType, int dimensions) {
		char[] desc = componentType.getType().getDescriptor().toCharArray();
		char[] arrayDesc = new char[desc.length + dimensions];
		Arrays.fill(arrayDesc, '[');
		System.arraycopy(desc, 0, arrayDesc, dimensions, desc.length);
		
		return org.objectweb.asm.Type.getType(new String(arrayDesc));
	}
	
	/**
	 * Converts a class name into an internal name.
	 *
	 * @param className
	 * @return
	 */
	@WrapperMember
	public static String getInternalName(String className) {
		return className.replaceAll("\\.", "/");
	}
	
	/**
	 * Converts a class name into a type descriptor.
	 *
	 * @param className
	 * @return
	 */
	@WrapperMember
	public static String getTypeDescriptor(String className) {
		return "L" + className.replaceAll("\\.", "/") + ";";
	}
	
	/**
	 * Converts an internal name into a class name.
	 *
	 * @param className
	 * @return
	 */
	@WrapperMember
	public static String getClassName(String internalName) {
		return internalName.replaceAll("/", ".");
	}
	
	public static Type getReturnType(String descriptor) {
		return WrapperRecycler.wrap(org.objectweb.asm.Type.getReturnType(descriptor), Type.class);
	}

	public static Type[] getArgumentTypes(String descriptor) {
		org.objectweb.asm.Type[] types = org.objectweb.asm.Type.getArgumentTypes(descriptor);
		Type[] array = new Type[types.length];
		for (int i = 0; i < array.length; i++)
			array[i] = WrapperRecycler.wrap(types[i], Type.class);
		return array;
	}
}
